"use strict";

const fs = require("fs");
const path = require("path");
const util_1 = require("./util");
const promisifed_fs_1 = require("./promisifed-fs");
const codeSign_1 = require("./codeSign");
const promise_1 = require("./promise");
const events_1 = require("events");
const bluebird_1 = require("bluebird");
const awaiter_1 = require("./awaiter");
const packager = require("electron-packager-tf");
const deb_1 = require("./deb");
const __awaiter = awaiter_1.tsAwaiter;
Array.isArray(__awaiter);
function addHandler(emitter, event, handler) {
    emitter.on(event, handler);
}
class Packager {
    constructor(options) {
        let repositoryInfo = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

        this.options = options;
        this.repositoryInfo = repositoryInfo;
        this.isTwoPackageJsonProjectLayoutUsed = true;
        this.eventEmitter = new events_1.EventEmitter();
        this.projectDir = options.projectDir == null ? process.cwd() : path.resolve(options.projectDir);
        this.appDir = this.computeAppDirectory();
    }
    artifactCreated(handler) {
        addHandler(this.eventEmitter, "artifactCreated", handler);
        return this;
    }
    dispatchArtifactCreated(path) {
        this.eventEmitter.emit("artifactCreated", path);
    }
    get devPackageFile() {
        return path.join(this.projectDir, "package.json");
    }
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            const buildPackageFile = this.devPackageFile;
            const appPackageFile = this.projectDir === this.appDir ? buildPackageFile : path.join(this.appDir, "package.json");
            yield bluebird_1.Promise.all(Array.from(new Set([buildPackageFile, appPackageFile]), promisifed_fs_1.parseJsonFile)).then(result => {
                this.metadata = result[result.length - 1];
                this.devMetadata = result[0];
                this.checkMetadata(appPackageFile);
                this.electronVersion = util_1.getElectronVersion(this.devMetadata, buildPackageFile);
            });
            const cleanupTasks = [];
            return promise_1.executeFinally(this.doBuild(cleanupTasks), error => promise_1.all(cleanupTasks.map(it => it())));
        });
    }
    doBuild(cleanupTasks) {
        return __awaiter(this, void 0, void 0, function* () {
            const platforms = this.options.platform === "all" ? getSupportedPlatforms() : [this.options.platform || process.platform];
            let macCodeSigningInfo = null;
            let keychainName = null;
            const distTasks = [];
            for (let platform of platforms) {
                const isMac = platform === "darwin";
                const archs = isMac ? ["x64"] : this.options.arch == null || this.options.arch === "all" ? ["ia32", "x64"] : [this.options.arch];
                for (let arch of archs) {
                    yield this.installAppDependencies(arch);
                    this.outDir = path.join(this.projectDir, "dist", this.metadata.name + "-" + platform + "-" + arch);
                    if (isMac) {
                        if (keychainName == null && this.options.cscLink != null && this.options.cscKeyPassword != null) {
                            keychainName = codeSign_1.generateKeychainName();
                            cleanupTasks.push(() => codeSign_1.deleteKeychain(keychainName));
                            yield bluebird_1.Promise.all([this.pack(platform, arch), codeSign_1.createKeychain(keychainName, this.options.cscLink, this.options.cscKeyPassword).then(it => macCodeSigningInfo = it)]);
                        } else {
                            yield this.pack(platform, arch);
                        }
                        yield this.signMac(path.join(this.outDir, this.metadata.name + ".app"), macCodeSigningInfo);
                    } else if (this.options.dist && platform === "win32") {
                        const installerOut = this.outDir + "-installer";
                        util_1.log("Removing %s", installerOut);
                        yield bluebird_1.Promise.all([this.pack(platform, arch), promisifed_fs_1.deleteDirectory(installerOut)]);
                    } else {
                        yield this.pack(platform, arch);
                    }
                    if (this.options.dist) {
                        distTasks.push(this.packageInDistributableFormat(platform, arch));
                        if (isMac) {
                            distTasks.push(this.zipMacApp().then(it => this.dispatchArtifactCreated(it)));
                        }
                    }
                }
            }
            return yield bluebird_1.Promise.all(distTasks);
        });
    }
    signMac(distPath, codeSigningInfo) {
        if (codeSigningInfo == null) {
            codeSigningInfo = { cscName: this.options.sign || process.env.CSC_NAME };
        }
        if (codeSigningInfo.cscName == null) {
            util_1.log("App is not signed: CSC_LINK or CSC_NAME are not specified");
            return Promise.resolve();
        } else {
            util_1.log("Signing app");
            return codeSign_1.sign(distPath, codeSigningInfo);
        }
    }
    computeAppDirectory() {
        let customAppPath = this.options.appDir;
        let required = true;
        if (customAppPath == null) {
            customAppPath = util_1.DEFAULT_APP_DIR_NAME;
            required = false;
        }
        let absoluteAppPath = path.join(this.projectDir, customAppPath);
        try {
            fs.accessSync(absoluteAppPath);
        } catch (e) {
            if (required) {
                throw new Error(customAppPath + " doesn't exists, " + e.message);
            } else {
                this.isTwoPackageJsonProjectLayoutUsed = false;
                return this.projectDir;
            }
        }
        return absoluteAppPath;
    }
    checkMetadata(appPackageFile) {
        const reportError = missedFieldName => {
            throw new Error("Please specify '" + missedFieldName + "' in the application package.json ('" + appPackageFile + "')");
        };
        const metadata = this.metadata;
        if (metadata.name == null) {
            reportError("name");
        } else if (metadata.description == null) {
            reportError("description");
        } else if (metadata.version == null) {
            reportError("version");
        } else if (metadata.build == null) {
            throw new Error("Please specify 'build' configuration in the application package.json ('" + appPackageFile + "'), at least\n\n" + JSON.stringify({
                build: {
                    "app-bundle-id": "your.id",
                    "app-category-type": "your.app.category.type",
                    "iconUrl": "see https://github.com/develar/electron-complete-builder#in-short"
                }
            }, null, "  ") + "\n\n is required.\n");
        } else if (metadata.author == null) {
            reportError("author");
        }
    }
    pack(platform, arch) {
        return new bluebird_1.Promise((resolve, reject) => {
            const version = this.metadata.version;
            let buildVersion = version;
            const buildNumber = process.env.TRAVIS_BUILD_NUMBER || process.env.APPVEYOR_BUILD_NUMBER || process.env.CIRCLE_BUILD_NUM;
            if (buildNumber != null) {
                buildVersion += "." + buildNumber;
            }
            const options = Object.assign({
                dir: this.appDir,
                out: path.dirname(this.outDir),
                name: this.metadata.name,
                platform: platform,
                arch: arch,
                version: this.electronVersion,
                icon: path.join(this.projectDir, "build", "icon"),
                asar: true,
                overwrite: true,
                "app-version": version,
                "build-version": buildVersion,
                "version-string": {
                    CompanyName: this.metadata.author,
                    FileDescription: this.metadata.description,
                    ProductVersion: version,
                    FileVersion: buildVersion,
                    ProductName: this.metadata.name,
                    InternalName: this.metadata.name
                }
            }, this.metadata.build, { "tmpdir": false });
            delete options.iconUrl;
            packager(options, error => error == null ? resolve(null) : reject(error));
        });
    }
    installAppDependencies(arch) {
        if (this.isTwoPackageJsonProjectLayoutUsed) {
            return util_1.installDependencies(this.appDir, arch, this.electronVersion);
        } else {
            util_1.log("Skipping app dependencies installation because dev and app dependencies are not separated");
            return Promise.resolve(null);
        }
    }
    zipMacApp() {
        util_1.log("Creating ZIP for Squirrel.Mac");
        const appName = this.metadata.name;
        const resultPath = `${ appName }-${ this.metadata.version }-mac.zip`;
        return util_1.spawn("zip", ["-ryXq", resultPath, appName + ".app"], {
            cwd: this.outDir,
            stdio: "inherit"
        }).thenReturn(this.outDir + "/" + resultPath);
    }
    packageInDistributableFormat(platform, arch) {
        const buildMetadata = this.devMetadata.build;
        if (platform === "win32") {
            return this.packageWinInDistributableFormat(buildMetadata, arch);
        } else if (platform === "linux") {
            return this.packageLinuxInDistributableFormat(buildMetadata, arch);
        } else {
            return this.packageMacInDistributableFormat(buildMetadata);
        }
    }
    packageWinInDistributableFormat(buildMetadata, arch) {
        return __awaiter(this, void 0, void 0, function* () {
            const customOptions = buildMetadata == null ? null : buildMetadata.win;
            let iconUrl = this.metadata.build.iconUrl;
            if (!iconUrl) {
                if (customOptions != null) {
                    iconUrl = customOptions.iconUrl;
                }
                if (!iconUrl) {
                    if (this.repositoryInfo != null) {
                        const info = yield this.repositoryInfo.getInfo(this);
                        if (info != null) {
                            iconUrl = `https://raw.githubusercontent.com/${ info.user }/${ info.project }/master/build/icon.ico`;
                        }
                    }
                    if (!iconUrl) {
                        throw new Error("iconUrl is not specified, please see https://github.com/develar/electron-complete-builder#in-short");
                    }
                }
            }
            const version = this.metadata.version;
            const outputDirectory = this.outDir + "-installer";
            const options = Object.assign({
                name: this.metadata.name,
                appDirectory: this.outDir,
                outputDirectory: outputDirectory,
                productName: this.metadata.name,
                version: version,
                description: this.metadata.description,
                authors: this.metadata.author,
                iconUrl: iconUrl,
                setupIcon: path.join(this.projectDir, "build", "icon.ico")
            }, customOptions);
            try {
                yield new bluebird_1.Promise((resolve, reject) => {
                    require("electron-winstaller-temp-fork").build(options, error => error == null ? resolve(null) : reject(error));
                });
            } catch (e) {
                if (e.message.indexOf("Unable to set icon") < 0) {
                    throw e;
                } else {
                    let fileInfo;
                    try {
                        fileInfo = yield promisifed_fs_1.stat(options.setupIcon);
                    } catch (e) {
                        throw new Error("Please specify correct setupIcon, file " + options.setupIcon + " not found");
                    }
                    if (fileInfo.isDirectory()) {
                        throw new Error("Please specify correct setupIcon, " + options.setupIcon + " is a directory");
                    }
                }
            }
            const appName = this.metadata.name;
            const archSuffix = arch === "x64" ? "-x64" : "";
            return yield Promise.all([promisifed_fs_1.renameFile(path.join(outputDirectory, appName + "Setup.exe"), path.join(outputDirectory, appName + "Setup-" + version + archSuffix + ".exe")).then(it => this.dispatchArtifactCreated(it)), promisifed_fs_1.renameFile(path.join(outputDirectory, appName + "-" + version + "-full.nupkg"), path.join(outputDirectory, appName + "-" + version + archSuffix + "-full.nupkg")).then(it => this.dispatchArtifactCreated(it))]);
        });
    }
    packageMacInDistributableFormat(buildMetadata) {
        const artifactPath = path.join(this.outDir, this.metadata.name + "-" + this.metadata.version + ".dmg");
        return new bluebird_1.Promise((resolve, reject) => {
            util_1.log("Creating DMG");
            const specification = {
                title: this.metadata.name,
                icon: "build/icon.icns",
                "icon-size": 80,
                background: "build/background.png",
                contents: [{
                    "x": 410, "y": 220, "type": "link", "path": "/Applications"
                }, {
                    "x": 130, "y": 220, "type": "file"
                }]
            };
            if (buildMetadata != null && buildMetadata.osx != null) {
                Object.assign(specification, buildMetadata.osx);
            }
            if (specification.title == null) {
                specification.title = this.metadata.name;
            }
            specification.contents[1].path = path.join(this.outDir, this.metadata.name + ".app");
            const appDmg = require("appdmg");
            const emitter = appDmg({
                target: artifactPath,
                basepath: this.projectDir,
                specification: specification
            });
            emitter.on("error", reject);
            emitter.on("finish", () => resolve());
        }).then(() => this.dispatchArtifactCreated(artifactPath));
    }
    packageLinuxInDistributableFormat(buildMetadata, arch) {
        const specification = {
            version: this.metadata.version,
            packageName: this.metadata.name,
            packageDescription: this.metadata.description,
            installPath: "/opt/" + this.metadata.name,
            maintainer: this.metadata.author,
            architecture: arch
        };
        if (buildMetadata != null && buildMetadata.linux != null) {
            Object.assign(specification, buildMetadata.linux);
        }
        return deb_1.makeDeb(specification, this.outDir).then(it => this.dispatchArtifactCreated(it));
    }
}
exports.Packager = Packager;
function getSupportedPlatforms() {
    if (process.platform === "darwin") {
        return ["darwin", "win32", "linux"];
    } else if (process.platform === "win32") {
        return ["win32"];
    } else {
        return ["linux", "win32"];
    }
}
//# sourceMappingURL=packager.js.map
